package Ukagaka::Shiori;
use utf8;
use Ukagaka::Shiori::Engine;
local $| = 1; # No OUTPUT Caching

sub new{
	my $package = shift;
	my $enginename = shift || 'Perl';
	my $engineoption = shift;
	my $acceptversion = shift || 3.0;
	*IN = shift || *STDIN;
	*OUT = shift || *STDOUT;
	my $enginemodule = 'Ukagaka::Shiori::Engine::'.$enginename;
	eval 'use '.$enginemodule.';';
	die 'Engine '.$enginemodule.' is unavailable.',"\n",$@ if $@;
	my $engine = $enginemodule->new($engineoption);
	$engine->acceptversion = $acceptversion;
	my $hash = {
		engine => $engine
	};
	return bless $hash,$package;
}

my $shiolink_responce = {
	L => \&load,
	S => \&request,
	U => \&unload
};
my $shiolink_types = join '|',keys %$shiolink_responce;
my $shiolink_regexp = qr/^\*($shiolink_types):(.*)/;
my $statusstring = {
	200 => 'OK',
	204 => 'No Content',
	311 => 'Not Enough',
	312 => 'Advice',
	400 => 'Bad Request',
	500 => 'Internal Server Error'
};

sub println;

sub mainloop{
	my $self = shift;
	$self->engine->init();
	my @requests;
	while(<IN>){
		chomp;
		if(my ($name,$value) = /$shiolink_regexp/){
			$shiolink_responce->{$name}->($self,$name,$value);
		}
	}
	$self->engine->end();
}

# MAIN ROUTINES
sub load{
	my $self = shift;
	$self->engine->loaddir = (shift,shift)[1];
}

sub unload{
	my $self = shift;
	exit;
}

sub request{
	my $self = shift;
	println '*' . shift . ':' . shift;
	my @requests;
	while(<STDIN>){
		chomp;
		if($_){
			push @requests,$_;
		}else{
			my ($method,$type,$version,$request,$statuscode,$responce);
			($method,$type,$version,$request) = $self->parse(@requests);
			($type,$version,$statuscode,$responce) = $self->callback($method,$type,$version,$request);
			$self->responce($type,$version,$statuscode,$responce);
			last;
		}
	}
}

# SUB ROUTINES

sub println{
	print OUT @_,"\n";
}

sub parse{
	my $self = shift;
	my $commandline = shift;
	my @requestheaders = @_;
	my ($method,$type,$version) = $commandline =~ m@^(.*) (SHIORI|SAORI)/(.*)@;
	my $request;
	$request->{$_->[0]} = $_->[1] for map {[/^(.+?): (.*)$/]} @requestheaders;
	return $method,$type,$version,$request;
}

sub callback{
	my $self = shift;
	my $method = shift;
	my $request_type = shift;
	my $request_version = shift;
	my $request = shift;
	my ($type,$version,$statuscode,$responce) = $self->engine->callback($method,$request_type,$request_version,$request);
	return $type,$version,$statuscode,$responce;
}

sub responce{
	my $self = shift;
	my $type = shift;
	my $version = shift;
	my $statuscode = shift;
	my $responce = shift;
	$responce->{$_} =~ s/[\n\r]//g for keys %$responce;
	println $type,'/',$version,' ',$statuscode,' ',$statusstring->{$statuscode};
	println $_,': ',$responce->{$_} for keys %$responce;
	println '';
}

# Accesser

sub engine{shift->{engine}}

1;
