package Ukagaka::Shiori::Engine::Kawari::Dict::Value::Analize;
use utf8;
use Ukagaka::Shiori::Engine::Kawari::Dict::Value::Tree;

sub new{
	my $package = shift;
	my $type = shift || 'root';
	my $rootvaluetree = Ukagaka::Shiori::Engine::Kawari::Dict::Value::Tree->new($type);
	my $hash = {
		value => $rootvaluetree,
		treestack => [],
		qstack => [],
		bratype => {
			'' => '',
			'(' => 'plain',
			'$(' => 'cmd',
			'${' => 'call',
			'$[' => 'expr'
		},
		bramatch => qr/\(|\$[\({\[]/,
		cketmatch => {
			'' => qr/[,)]/,
			'plain' => qr/\)/,
#			'cmd' => qr/\)/,
			'cmd' => qr/[;)]/,
			'call' => qr/\}/,
			'expr' => qr/\]/
		}
	};
	my $self = bless $hash,$package;
	$self->pushtreestack($self->{value});
	my $valuetree = Ukagaka::Shiori::Engine::Kawari::Dict::Value::Tree->new('');
	$self->lasttreestack()->pushvalue($valuetree);
	$self->pushtreestack($self->lasttreestack()->lastvalue());
	return $self;
}
=pod
valuetree => [
	"aaa",
	{ <- $treestackに格納
		type => "plain"
		valuetree => [
			"aaa"
			"  bbb\n"
			{}
		]
	}
]
=cut

sub getvalue{
	my $self = shift;
	return $self->{value};
}

sub analize{
	my $self = shift;
	my $value = shift;
	$self->build_valuetree($value);
	return $self->stackdepth() ? undef : 1;
}

sub enquote{
	my $self = shift;
	my $value = shift;
	while($value =~ s/(["'])(.*?)(?<![^\\]\\)\1/$self->nullquote($self->qcount())/e){#" "",''(後方はエスケープを回避)を\0\d\0に置換
		push @{$self->{qstack}},$1.$2.$1;
	}
	return $value;
}

sub dequote{
	my $self = shift;
	my $value = shift;
	$value =~ s/\0([0-9]+?)\0/$self->{qstack}->[$1]/g;# \0\d\0を元の"",''に置換
	return $value;
}

sub qcount{
	my $self = shift;
	return $#{$self->{qstack}} + 1;
}

sub nullquote{
	my $self = shift;
	my $value = shift;
	return "\0" . $value . "\0";
}

sub build_valuetree{
	my $self = shift;
	my $value = shift;
	$value = $self->enquote($value);
	while($value){
		my ($pre_bracket,$bracket,$post_bracket) = $value =~ /
			(.*?) # 括弧の前 $1
			( # 括弧の種類判別 $2
				$self->{bramatch}|$self->{cketmatch}->{$self->lasttreestack()->{type}}
			)
			(.*) # 括弧の後 $3
		/xs;# エスケープされていない開き括弧の前後.*を読み取り
		#)})))}
		if(defined $bracket){ # 括弧がある場合:トークンに括弧前部分/次回括弧判定に括弧後部分
			$self->lasttreestack()->pushvalue($self->dequote($pre_bracket));
			$value = $post_bracket;
			if($self->{bratype}->{$bracket}){
				my $valuetree = Ukagaka::Shiori::Engine::Kawari::Dict::Value::Tree->new($self->{bratype}->{$bracket});
				$self->lasttreestack()->pushvalue($valuetree);
				$self->pushtreestack($self->lasttreestack()->lastvalue());
#print STDERR "  ",("  " x $self->stackdepth()),"bra pre  =$pre_bracket\n";###
#print STDERR "  ",("  " x $self->stackdepth()),"bra      =$bracket\n";###
#print STDERR "  ",("  " x $self->stackdepth()),"bra post =$post_bracket\n";###
			}elsif($bracket eq ';'){
				# 案1:これだと$(aa;aa$(n))->$(aa)$(aa$(n))に復元される。意味は割と同じだが……。復元するにはセパレータでも挟むしかない。
				# Value::Treeの文字列化と数値化のオーバーロードをそれぞれ文字列化と解釈後文字列化に割り当てればあるいは……。
				# 有力
				$self->poptreestack();
				# my $cmdsepvaluetree = Ukagaka::Shiori::Engine::Kawari::Dict::Value::Tree->new('cmdsep');
				my $cmdvaluetree = Ukagaka::Shiori::Engine::Kawari::Dict::Value::Tree->new('cmd');
				# $self->lasttreestack()->pushvalue($cmdsepvaluetree);
				$self->lasttreestack()->pushvalue($cmdvaluetree);
				$self->pushtreestack($self->{value}->lastvalue());
				# 案2:ここでは何もしない。つまりプレーンな(コマンド)文字列が連続する。;の復元には前方後方参照して始終端かオブジェクトでないことを確認する作業が必要
			}elsif($bracket eq ','){
				$self->poptreestack();
				my $valuetree = Ukagaka::Shiori::Engine::Kawari::Dict::Value::Tree->new('');
				$self->{value}->pushvalue($valuetree);
				$self->pushtreestack($self->{value}->lastvalue());
			}else{
#print STDERR "  ",("  " x $self->stackdepth()),"cket pre =$pre_bracket\n";###
#print STDERR "  ",("  " x $self->stackdepth()),"cket     =$bracket\n";###
#print STDERR "  ",("  " x $self->stackdepth()),"cket post=$post_bracket\n";###
				$self->poptreestack();
			}
		#)})}
		}else{ # 括弧がない場合:トークンに全体/判定終了
			$self->lasttreestack()->pushvalue($self->dequote($value));
			undef $value;
		}
		#{{
		die "Unmatch bracket." if $self->stackdepth() < 0;
	}
}

sub pushtreestack{
	my $self = shift;
	my @value = @_;
	push @{$self->{treestack}},@value;
}
sub poptreestack{
	my $self = shift;
	pop @{$self->{treestack}};
}
sub lasttreestack{
	my $self = shift;
	return $self->{treestack}->[-1];
}
sub stackdepth{
	my $self = shift;
	return $#{$self->{treestack}} - 1;
}

1;
