package Ukagaka::Shiori::Engine::Perl;
use utf8;
use Ukagaka::Shiori::Engine;
use Ukagaka::Shiori::Engine::Perl::Events;

our @ISA = qw(Ukagaka::Shiori::Engine);

sub new{
	my $package = shift;
	my $option = shift;
	my $eventspackage = $option->{events} || 'Ukagaka::Shiori::Engine::Perl::Events';
	die 'Please inherit Ukagaka::Shiori::Engine::Perl::Events',"\n",'Package: ',$eventspackage unless $eventspackage && $eventspackage->isa('Ukagaka::Shiori::Engine::Perl::Events');
	my $events = $eventspackage->new();
	$option->{securitylevel} = 'local' unless $option->{securitylevel};
	my $data = {
		shiori => {
			version => '1.0',
			craftman => '奈良阪某',
			craftmanw => 'Narazaka',
			shioriid => 'Shiolink_Perl_ShioriEngine_Perl',
			name => 'Shiolink/Perl ShioriEngine_Perl'
		},
		events => $events
	};
	my $self = Ukagaka::Shiori::Engine->new($option,$data);
	return bless $self,$package;
}

my $events = sub{shift->(undef,'events')}; # works like private method -> events cannot overwrite

sub callback{ # only SHIORI not SAORI
	my $self = shift;
	my $method = shift;
	my $type = shift;
	my $version = shift;
	my $request = shift;
	my $statuscode = 204;
	my $responce = {};
	my $securitylevel = $request->{SecurityLevel};
	# どう見てもkawari_shiori.cppのベタ移植です。本当にありがとうございました。
	if($version eq '3.0'){ # SHIORI/3.0
		my $id = $request->{ID};
		$id =~ s/\./_/g;
		if($method eq 'GET'){ # GET
			my ($value,$headers);
			($value,$statuscode,$headers) = $self->$events->$id($request); # Call method:{ID}
			$responce->{Value} = $value;
			if(ref $headers){ # for OnCommunicate etc.
				$responce->{$_} = $headers->{$_} for keys %$headers;
			}
			if($responce->{Value} eq ''){
				if($request->{ID} eq 'version'){
					$responce->{Value} = $self->shiori->{version};
				}elsif($request->{ID} eq 'craftman'){
					$responce->{Value} = $self->shiori->{craftman};
				}elsif($request->{ID} eq 'craftmanw'){
					$responce->{Value} = $self->shiori->{craftmanw};
				}elsif($request->{ID} eq 'shioriid'){
					$responce->{Value} = $self->shiori->{shioriid};
				}elsif($request->{ID} eq 'name'){
					$responce->{Value} = $self->shiori->{name};
				}
			}
			$statuscode = $responce->{Value} ne '' ? 200 : 204 unless $statuscode;
		}elsif($method eq 'NOTIFY'){
			$self->header->{Sender} = $request->{Reference0} if $id eq 'ownerghostname'; # ownerghostnameを保存し、以後のSenderに使う
			$self->$events->$id($request); # Call method:{ID}
			$statuscode = 204;
		}
	}elsif($version >= 2 && $version < 3){ # Not implemented
		if(my ($command) = $method =~ /GET (.+)/){
			if($command eq 'Version'){ # SHIORI/2.0 SHIORI/2.6
				$version = $self->acceptversion;
				$responce->{ID} = $self->shiori->{shioriid};
				$responce->{Craftman} = $self->shiori->{craftman};
				$responce->{Version} = $self->shiori->{version};
				$statuscode = 200;
			}elsif($command eq 'Sentence'){ # SHIORI/2.0 SHIORI/2.2(local/SSTP) SHIORI/2.3b(SSTP) SHIORI/2.6
				my ($sentence,$headers);
				if($request->{Sentence} ne ''){ # SHIORI/2.0 SHIORI/2.3b SHIORI/2.6 Sentenceヘッダに話しかけられた内容
					($sentence,$statuscode,$headers) = $self->$events->Sentence($request); # Call method:Sentence
				}else{ # SHIORI/2.2 SHIORI/2.6 Eventヘッダにイベント
					my $event = $request->{Event};
					$event =~ s/\./_/g;
					($sentence,$statuscode,$headers) = $self->$events->$event($request); # Call method:{Event}
				}
				if(ref $headers){
					$responce->{$_} = $headers->{$_} for keys %$headers;
				}
				$responce->{Sentence} = $sentence;
				$statuscode = $responce->{Sentence} ? 200 : 204;
				# ゴースト間コミュニケート # SHIORI/2.3b SHIORI/2.6
				unless($responce->{To} eq 'stop' || (
					$responce->{To} eq '' && (
						$request->{Age} eq '' ||
						$request->{Sender} eq '' ||
						$request->{Sender} eq 'User' ||
						$request->{Sender} eq 'Nobody'
					)
				)){ # コミュニケート終了条件否定
					unless($responce->{To} ne '' && $responce->{To} ne $request->{Sender}){ # 新しい相手にコミュニケートする条件否定
						$responce->{To} = $request->{Sender};
						$responce->{Age} = $request->{Age} eq '' ? 0 : $request->{Age} + 1;
					}
				}
			}elsif($command eq 'String'){ # SHIORI/2.5 IDヘッダにリソースID
				my $id = $request->{ID};
				$id =~ s/\./_/g;
				$responce->{String} = $self->$events->$id($request); # Call method:{ID}
				$statuscode = $responce->{String} ? 200 : 204;
			}elsif($command eq 'Word'){ # SHIORI/2.0 Typeヘッダに単語のタイプ
				$responce->{Word} = $self->$events->Word($request); # Call method:Word
				$statuscode = $responce->{Word} ? 200 : 204;
			}elsif($command eq 'Status'){ # SHIORI/2.0 SHIORI/2.6
				$responce->{Status} = $self->$events->Status($request); # Call method:Status
				$statuscode = $responce->{Status} ? 200 : 204;
			}
		}elsif(my ($id) = $method =~ /NOTIFY (.+)/){ # SHIORI/2.0 SHIORI/2.3b SHIORI/2.6
			if($id eq 'OwnerGhostName'){ # SHIORI/2.0 SHIORI/2.3b SHIORI/2.6
				$self->header->{Sender} = $request->{Ghost}; # 以後のSenderに使う
			}
			# ヘッダの一意性を前提とする現状の実装では対応不可
			#elsif($id eq 'OtherGhostName'){ # SHIORI/2.3b SHIORI/2.6
			#}
		}elsif($method eq 'TEACH'){ # SHIORI/2.4
			my $sentence;
			($sentence,$statuscode) = $self->$events->TEACH($request); # Call method:TEACH
		}
	}
	$responce->{Charset} = $self->header->{Charset};
	$responce->{Sender} = $self->header->{Sender} unless $responce->{Sender};
	return $type,$version,$statuscode,$responce;
}

1;
