# our $eventspackage にイベント応答を定義するパッケージ名を指定してください。
# init.plでのUkagaka::Shiori->newでの引数に使われ、Ukagaka::Shiori::Engine::Perlから呼ばれます。
our $eventspackage = 'Ukagaka::Shiori::Engine::Perl::Events::Simple';
# our $eventspackage に指定したパッケージ名でイベント応答を定義していきます。
# イベント応答定義パッケージは最初に
# use Ukagaka::Shiori::Engine::Perl::Events;
# our @ISA = ('Ukagaka::Shiori::Engine::Perl::Events');
# してUkagaka::Shiori::Engine::Perl::Eventsを継承しなければなりません。
# イベントやその他のメソッドは適宜オーバーライドしてください。
# 注意:イベント名で呼ばれるメソッド名の.は_に変換されています。
package Ukagaka::Shiori::Engine::Perl::Events::Simple;
use Ukagaka::Shiori::Engine::Perl::Events;
use utf8;
our @ISA = ('Ukagaka::Shiori::Engine::Perl::Events');

our $info; # 情報格納
our $dict; # 華和梨似の辞書データ格納

sub new{
	my $package = shift;
	loadkisdic();
	return bless {},$package;
}

# 備考：華和梨辞書似の辞書は便利なので実装例に加えていますが推奨というわけではないです。

# 華和梨似の辞書処理
sub loadkisdic{
	# 華和梨似の辞書を読む
	open my $fh,'<:utf8','./scripts/events-simple_kis_lesser.txt';
	while(<$fh>){
		chomp;
		# コメント、空白行スキップ
		next if /^[ \t]*(?:#|[ \t]*$)/;
		# ハッシュ格納
		my ($key,$value) = /^[ \t]*(.+?)[ \t]*:[ \t]*(.*)[ \t]*$/;
		if($value){
			my @valuelist = split /[ \t]*,[ \t]*/,$value;
			push @{$dict->{$key}},@valuelist;
		}
	}
	close $fh;
}

sub callsub{ # 華和梨似の辞書${}のパース
	my $key = shift;
	my $ret = $dict->{$key}->[rand ($#{$dict->{$key}} + 1)];
	$ret =~ s/\$\{([^\}]+)\}/callsub($1)/eg;
	return $ret;
}

sub speak{ # 華和梨似の辞書からさくらスクリプトを返す
	my $key = shift;
	my $ret = callsub $key;
#	$ret .= '\e'if $ret && $ret !~ /\\e$/; # リソース取得に困るのでコメントアウト
	return $ret;
}

# Perlの便利機能
sub AUTOLOAD{# Perlの機能 関数名がなかった場合AUTOLOADが呼ばれる
	our $AUTOLOAD; # $AUTOLOADを定義しておくと呼び出された関数名が入る
#	my @args = @_; # AUTOLOAD関数の引数には呼び出された引数が入る
	my $self = shift;
	my $pkgname = ref $self;
	$AUTOLOAD =~ s/^${pkgname}:://;
	return speak $AUTOLOAD; # 関数名(=イベント名)の華和梨似辞書を発話
}

# 渡された引数からランダムで1つ選んで改行を省いて返す関数
sub rr{
	my $num = rand $#_+1;
	my $ret = $_[$num];
	$ret =~ s/[\n\r]//g;
	return $ret;
}

# おすすめサイトリストなどのURLリスト書式整形されたデータを返す
# $urllist = [
#             {name => 'サイト名', url => 'http://サイトURL', banner => 'http://バナーURL'},
#             {name => '-', url => '', banner => ''},
#             ...
# ];
# を渡すとデータが帰ってくるのでそのままValueに返す。
sub urllist{
	my $urllist = shift;
	my $b1 = chr(1);
	my $b2 = chr(2);
	return join '',map {$_->{name}.$b1.$_->{url}.$b1.$_->{banner}.$b2} @$urllist;
}

# 各種イベント

# Perlはreturnがない場合最後に評価された文が返値となる。
# なおかつブロック({})中の最後の文には;がいらない。
sub OnBoot{'\0\s[0]\1\s[1]\0ゴーストが起動されました。\e'}

sub OnFirstBoot{
	# ''内を複数行にかく
	# 第一引数(Valueヘッダ)の改行はSHIORI応答側で自動的に削除してくれます
	'
\h\s[0]\u\s[10]\u
Shiolink/Perl\w6\n
 Powered by \_a[http://d.hatena.ne.jp/dot_station/20080528/p1]SHIOLINK\_a(\_a[http://code.google.com/p/shiori-basic/]shiori-basic\_a)\n
 Author: \_a[http://narazaka.net/]Narazaka\_a\w9\n\n[half]
 testing...
\hというわけで超暫定テスト版です。\w9\n
とりあえず\w4Perlで栞が動いてる感(ないけど)を味わってください。
\e
';
}

sub OnGhostChanging{
	return <<'EOM'; # ''内を複数行にかく構文(ヒアドキュメント)。どっちかっていうと""(<<"EOM")で使うことの方が多いけど。
\h\s[0]\u\s[10]
\uかわるよー\w9
\hかわるねー\w9
\e
EOM
}

sub OnGhostChanged{
	my $self = shift;
	my $request = shift; # リクエストヘッダのハッシュリファレンス
	if($request->{Reference0} eq $info->{ghostname}){
		return '\h\s[0]\u\s[10]\0最遊記リロード\e';
	}elsif($request->{Reference0} eq 'さくら'){
		return '\h\s[0]\u\s[10]\0さくらたんでした\e';
	}elsif($request->{Reference0} eq 'レイチェル'){
		return '\h\s[0]\u\s[10]\0猫がしゃべってた。\w9\1まあ。\e';
	}elsif($request->{Reference0} eq '○×△□'){
		return '\h\s[0]\u\s[10]\0どうも。\w9\1どうも。\e';
	}elsif($request->{Reference0} eq 'Emily'){
		return '\h\s[0]\u\s[10]\0SSPの主から交代\w9\e';
	}elsif($request->{Reference0} eq '和泉はるか'){
		return '\h\s[0]\u\s[10]\0ぼくは近畿日本鉄道ちゃん。\w9\1語呂悪い。\e';
	}elsif($request->{Reference0} eq 'ヒトラ氏'){
		return '\h\s[0]\u\s[10]\0うん。\e';
	}elsif($request->{Reference0} eq 'みるむ'){
		return '\h\s[0]\u\s[10]\0さて。\e';
	}elsif($request->{Reference0} eq '○|￣|_'){
		return '\h\s[0]\u\s[10]\0絶望しました。\w9\1ああ。\e';
	}elsif($request->{Reference0} eq '(・∀・)'){
		return '\h\s[0]\u\s[10]\0いい。\e';
	}elsif($request->{Reference0} eq '美耳'){
		return '\h\s[0]\u\s[10]\1せやな。\w9\0なにが？\e';
	}elsif($request->{Reference0} eq 'ＡＩ'){
		return '\h\s[0]\u\s[10]\0AIとま\e';
	}elsif($request->{Reference0} eq '奎子'){
		return '\h\s[0]\u\s[10]\1流石まんじろう先生や。\w9\0はあ。\e';
	}elsif($request->{Reference0} eq 'サクラ'){
		return '\h\s[0]\u\s[10]\0さくらとサクラってあれかな、\w5ツバサ？\w9\1いや……。\e';
	}elsif($request->{Reference0} eq '楔'){
		return '\h\s[0]\u\s[10]\0庵野か……\w4なにもかもなつかしいね。\w9\1お前そう言うような世代でもないやろ。\e';
	}elsif($request->{Reference0} eq '美紅'){
		return '\h\s[0]\u\s[10]\0謎。\w9\1謎か。\e';
	}elsif($request->{Reference0} eq '電源ボたん'){
		return '\h\s[0]\u\s[10]\0Perl関係で語尾に「たん」……。\w9\1CTANはTeXだしな。\w9\0\n\n[half]……。\e';
	}elsif($request->{Reference0} eq '街っぽい'){
		return '\h\s[0]\u\s[10]\0A列車派です。\w9\1\e';
	}elsif($request->{Reference0} eq 'めもりーな'){
		return '\h\s[0]\u\s[10]\1DDR?-???とPC?-????を混在させないでいただきたい。\w9\0しょうがない。\e';
	}elsif($request->{Reference0} eq 'わはー'){
		return '\h\s[0]\u\s[10]\0わはー\e';
	}elsif($request->{Reference0} eq 'ククリ'){
		return '\h\s[0]\u\s[10]\0グルグルか……\w4なにもかもなつかしいね。\w9\1まあ。\e';
	}
	#ランダムに呼ばれる returnはいらないけどつけとく
	return rr '\h\s[0]\u\s[10]\0かわった\e','\h\s[0]\u\s[10]\0OnGhostChangedです\e';
}

sub OnClose{
	# 華和梨似辞書のエントリをよんでいる
	# returnいらず
	speak 'OnClose';
}

sub OnCloseAll{
	# 他のイベントと全く同じ処理をさせたい場合はPerlのgoto文(Cのとは意味合いが違う)が便利
	goto &OnClose;
}

our $aitalkcount = 0;

sub OnSecondChange{ # 超雑なAI発話処理
	my $self = shift;
	my $request = shift; # リクエストヘッダのハッシュリファレンス
	return if $request->{Reference1}; # 見切れフラグが立っていれば何も返さず処理を中断
	if($request->{Reference2} == 1){ # 重なりフラグが立っていれば何も返さず処理を中断
		return;
	}
	$aitalkcount++; # カウントを秒ごとに増やす
	if($aitalkcount >= 60){ # カウントが60になったら(1分たったら)
		OnAITalk();
	}
}

sub OnMouseDoubleClick{
	OnAITalk();
}

sub OnAITalk{
	$aitalkcount = 0; # カウントを戻す
	return speak 'sentence'; # AITalk(sentenceエントリ)を発話する
}

sub ownerghostname{
	my $self = shift;
	my $request = shift;
	$info->{ghostname} = $request->{Reference0};
}

sub sakura_portalsites{ # IDの.は_に置換
	return urllist [
		{name => 'Shiolink/Perl Simple配布元', url => 'http://narazaka.net/c/ukagaka/ghost/shiolink_perl_simple/', banner => 'http://narazaka.net/banner.png'},
		{name => '-', url => '', banner => ''},
		{name => 'SHIOLINK.dll概要', url => 'http://d.hatena.ne.jp/dot_station/20080528/p1', banner => ''},
		{name => 'SHIOLINK.dllダウンロード', url => 'http://code.google.com/p/shiori-basic/', banner => ''},
		{name => 'Strawberry Perl', url => 'http://strawberryperl.com/', banner => ''},
		{name => '-', url => '', banner => ''},
		{name => 'The Perl Programming Language', url => 'http://www.perl.org/', banner => ''}
	];
}

sub sakura_recommendsites{
	return urllist [
		{name => 'PerlIO', url => 'http://www.rwds.net/kuroita/program/Perl_unicode.html', banner => ''},
		{name => 'Mojolicious', url => 'http://d.hatena.ne.jp/perlcodesample/20110402/1305602329', banner => ''},
		{name => '-', url => '', banner => ''},
		{name => '．さくらStation', url => 'http://www.nanican.net/dot-sakura/', banner => ''},
		{name => '-', url => '', banner => ''},
		{name => 'プログラマブル準AI 華和梨', url => 'http://kawari.sourceforge.net/', banner => ''},
		{name => '里々', url => 'http://www.geocities.jp/poskoma/', banner => ''},
		{name => 'Yet Another AYA', url => 'http://code.google.com/p/yaya-shiori/', banner => ''},
		{name => '美坂', url => 'http://lovelove.rabi-en-rose.net/index_misaka.php', banner => ''},
		{name => '結奈 SecondEdition', url => 'http://www38.atwiki.jp/shiori_yuhna-se/', banner => ''},
		{name => 'カスタマイズ可能 AIc "真絵"', url => 'http://efasus.sakura.tv/', banner => ''},
		{name => 'ese-shiori', url => 'http://web.archive.org/web/20060216054722/http://nagoya.cool.ne.jp/tmizu/', banner => ''},
		{name => '梨野', url => 'http://www.junkyard.jp/nasino/index.shtml', banner => ''},
		{name => '汎用ＡＩ翡翠', url => 'http://www2.tba.t-com.ne.jp/HIROHIRO/hisukoha.htm', banner => ''},
		{name => 'Psyche System', url => 'http://www.aa.alpha-net.ne.jp/shinra08/ghost/', banner => ''},
		{name => '里珠/Proxy', url => 'http://www.mikage.to/PerlShiori/', banner => ''},
		{name => 'Scheme栞', url => 'http://spoon.if.land.to/file/saihaifu/kasumigabou/saihaihu_kasumigabou_ki.htm', banner => ''},
		{name => 'Lisp栞-for-win32', url => 'http://www.geocities.jp/isson40/', banner => ''},
		{name => 'なでしこ栞', url => 'http://spoon.if.land.to/file/saihaifu/muku_page/saihaihu_muku.htm', banner => ''},
		{name => '紅蓮(Ruby栞)', url => 'http://www12.atpages.jp/narup/log/page001.htm#35', banner => ''},
		{name => '大蛇(Python栞)', url => 'http://www12.atpages.jp/narup/log/page001.htm#36', banner => ''},
		{name => 'D.N.Proxy', url => 'http://www12.atpages.jp/narup/log/page002.htm#309', banner => ''}
	];
}

sub kero_recommendsites{
	return urllist [
		{name => '難読駅名クイズ', url => 'http://narazaka.net/r/eki/', banner => 'http://narazaka.net/banner.png'}
	];
}


sub OnAnchorSelect{ # アンカー選択時の処理
	my $self = shift;
	my $request = shift;
	return '\j['.$request->{Reference0}.']';
}

sub OnUpdateBegin{'\0ネットワーク更新を開始します。\w9\e'}
sub OnUpdateReady{
	my $self = shift;
	my $request = shift;
	'\_q\1Update '.($request->{Reference0}+1).' files\_q\w6\e'
}
sub OnUpdate_OnDownloadBegin{
	my $self = shift;
	my $request = shift;
	'\_q\1Downloading ['.$request->{Reference0}.']\n\n[half]['.progressbar($request->{Reference1},$request->{Reference2}+1).']\_q\e'
}
sub OnUpdateComplete{
	my $self = shift;
	my $request = shift;
	if($request->{Reference0} eq 'none'){
		'\0ローカルのファイルは最新です。\w9\e'
	}else{
		'\0更新は正常終了しました。\w9\e'
	}
}
sub OnUpdateFailure{
	my $self = shift;
	my $request = shift;
	my $ret = '\_q';
	if($request->{Reference0} eq 'timeout'){
		$ret .= '\1Timeout.';
	}elsif($request->{Reference0} eq 'md5 miss'){
		$ret .= '\1MD5 Checksum Compare Failure.';
	}elsif($request->{Reference0} eq 'too slow'){
		$ret .= '\1Transport too slow.';
	}elsif($request->{Reference0} eq 'artificial'){
		$ret .= '\1User stop.';
	}elsif($request->{Reference0} eq 'fileio'){
		$ret .= '\1File system IO error.';
	}elsif($request->{Reference0} eq 'readonly'){
		$ret .= '\1File system is readonly.';
	}elsif($request->{Reference0} eq '404'){
		$ret .= '\1404 File Not Found.';
	}else{
		$ret .= '\1Unknown error Status:'.$request->{Reference0}.'';
	}
	$ret .= '\nFile ['.$request->{Reference1}.']' if $request->{Reference1};
	$ret .= '\_q\0更新は失敗しました。\w9\e';
	return $ret;
}
sub progressbar{
	my $now = shift;
	my $all = shift;
	my $per = int 20 * $now / $all;
	return '#' x $per . ' ' x (20 - $per);
}

sub OnNarCreating{'\1\_qCreating nar archive...\_q\e'}
sub OnNarCreated{'\1\_qCreating nar archive...\n\n[half]Done.\_q\e'}
sub OnUpdatedataCreating{'\1\_qCreating update information...\_q\e'}
sub OnUpdatedataCreated{'\1\_qCreating update information...\n\n[half]Done.\_q\e'}

1;
